using System;
using System.Collections.Generic;
using System.Text;

using ManyLeaves.Infrastructure.Interface;

namespace AKSYSTech.Device.Adapters.Interface
{
	public interface IDevice : IDisposable
	{
		#region Events

		// need an event for (at least) every Property change

		event EventHandler<EventArgs<Position[]>> PositionChanged;
		event EventHandler<EventArgs<DeviceState>> DeviceStateChanged;
		event EventHandler<EventArgs<AutoState>> AutoStateChanged;
		event EventHandler<EventArgs<GCodeState>> GCodeStateChanged;
		event EventHandler<EventArgs<String>> StatusMessageChanged;
		event EventHandler<EventArgs<Rate[]>> RateChanged;
        event EventHandler<EventArgs<Switch[]>> SwitchChanged;

		#endregion

		#region Properties

		String GCode { set; }
		AutoState AutoState { set; }

		#endregion

		#region Methods

		void Load();
		void Close();

		void Rate(RateName name, Double value);
        void Rate(RateName name, JogDirection direction);

        Double Min(RateName name);
        Double Max(RateName name);

        void Switch(SwitchName name, SwitchPosition position);

       	void Jog(Axis axis, JogDirection direction);

		#endregion
	}
}
