using System;
using System.Collections.Generic;
using System.Text;
using System.IO;

using System.Threading;
using System.Runtime.InteropServices;

using AKSYSTech.Device.Adapters.Interface;

using ManyLeaves.Infrastructure.Interface;

namespace AKSYSTech.Device.Adapters
{
	internal class Background : IDisposable
	{
		#region Events

		public event EventHandler<EventArgs<Position[]>> PositionChanged;
		public event EventHandler<EventArgs<AutoState>> AutoStateChanged;
		public event EventHandler<EventArgs<GCodeState>> GCodeStateChanged;
		public event EventHandler<EventArgs<String>> StatusMessageChanged;
        public event EventHandler<EventArgs<Rate[]>> RateChanged;
        public event EventHandler<EventArgs<Switch[]>> SwitchChanged;

		#endregion

		private Thread _thread = null;

		private ManualResetEvent _done = null;

		private AutoState _autoState = AutoState.Stopped;
        private GCodeState _gcodeState = GCodeState.Completed;

		public Background(Mach mach)
		{
			_done = new ManualResetEvent(false);
			_thread = new Thread(new System.Threading.ParameterizedThreadStart(DoWork));
			_thread.IsBackground = true;
			_thread.Start(mach);
		}

		public void Dispose()
		{
			_done.Set();
			_thread.Join();
		}

		private void DoWork(Object o)
		{
			const Int32 __delay = 50;

			// we'll need a variable to hold the current value(s) for each xxxChanged event we fire

			Double __xValue = Double.MaxValue;
            Double __yValue = Double.MaxValue;
            Double __zValue = Double.MaxValue;
            Double __aValue = Double.MaxValue;

			AutoState __autoState = _autoState;
			GCodeState __gcodeState = _gcodeState;
			
			String __statusMessage = String.Empty;

            Double __feedRateValue = 0;
            Double __rapidRateValue = 0;
            Double __spindleRateValue = 0;

            SwitchPosition __spindleValue = SwitchPosition.Off;
            SwitchPosition __coolantValue = SwitchPosition.Off;

			Boolean __isLoading = false;
			Boolean __isMoving = false;

			Mach __mach = (Mach)o;

			while (!_done.WaitOne(__delay, false))
			{
				//String __message = String.Format
				//    (
				//    "AutoState = {0}, IsLoading = {1}, IsMoving = {2}, IsStopped = {3}, IsEstop = {4}\r\n",
				//    __autoState.ToString(),
				//    __mach.Scripter.IsLoading().ToString(),
				//    __mach.Scripter.IsMoving().ToString(),
				//    __mach.Scripter.IsStopped().ToString(),
				//    __mach.Scripter.IsEstop().ToString()
				//    );
				//File.AppendAllText("Log.txt", __message);
                #region GCodeState
                if (__mach.Scripter.IsLoading() == 0)
				{
					if (__isLoading) // was loading
					{
						_gcodeState = GCodeState.Loaded;
					}
					__isLoading = false;
				}
				else
				{
					__isLoading = true;
				}
				if (__mach.Scripter.IsEstop() == 0)
				{
					if (__mach.Scripter.IsMoving() == 0)
					{
						if (__isMoving) // was moving
						{
							if (_autoState == AutoState.Running) // for Pause&Continue
							{
							    _gcodeState = GCodeState.Completed;
							    _autoState = AutoState.Ready;
							}
						}
						__isMoving = false;
					}
					else
					{
						if (!__isMoving) // wasn't moving
						{
                            _gcodeState = GCodeState.Running;
						}
						__isMoving = true;
					}
				}
				else
				{
					if (__isMoving) // was moving
					{
						_gcodeState = GCodeState.Aborted;
					}
					__isMoving = false;
                }
                #endregion
                #region PositionChanged
				if (this.PositionChanged != null)
				{
					List<Position> __positions = new List<Position>();
					Double __xValueNow = __mach.Scripter.GetDRO(0);
					Double __yValueNow = __mach.Scripter.GetDRO(1);
					Double __zValueNow = __mach.Scripter.GetDRO(2);
                    Double __aValueNow = __mach.Scripter.GetDRO(3);
					if (__xValue != __xValueNow)
					{
						__xValue = __xValueNow;
						__positions.Add(new Position(Axis.X, __xValue));
					}
					if (__yValue != __yValueNow)
					{
						__yValue = __yValueNow;
						__positions.Add(new Position(Axis.Y, __yValue));
					}
					if (__zValue != __zValueNow)
					{
						__zValue = __zValueNow;
						__positions.Add(new Position(Axis.Z, __zValue));
					}
                    if (__aValue != __aValueNow)
					{
						__aValue = __aValueNow;
						__positions.Add(new Position(Axis.A, __aValue));
					}
					if (__positions.Count != 0)
					{
						EventArgs<Position[]> __e = new EventArgs<Position[]>(__positions.ToArray());
						this.PositionChanged(__mach, __e);
					}
                }
                #endregion
                #region AutoStateChanged
                if (this.AutoStateChanged != null)
				{
					// can we infer autoState from a Scripter Method?
					// but remember we need to be able to support AutoState.Completed
					if (__autoState != _autoState)
					{
						__autoState = _autoState;
						EventArgs<AutoState> __e = new EventArgs<AutoState>(_autoState);
						this.AutoStateChanged(__mach, __e);
					}
				}
                #endregion
                #region GCodeStateChanged
                if (this.GCodeStateChanged != null)
				{
					if (__gcodeState != _gcodeState)
					{
						__gcodeState = _gcodeState;
						EventArgs<GCodeState> __e = new EventArgs<GCodeState>(_gcodeState);
						this.GCodeStateChanged(__mach, __e);
					}
				}
                #endregion
                #region StatusMessageChanged
                if (this.StatusMessageChanged != null)
				{
					String __statusMessageNow = __mach.Scripter.GetMessage();
					if (__statusMessage != __statusMessageNow)
					{
						__statusMessage = __statusMessageNow;
						EventArgs<String> __e = new EventArgs<String>(__statusMessage);
						this.StatusMessageChanged(__mach, __e);
					}
				}
                #endregion
                #region RateChanged
                if (this.RateChanged != null)
				{
                    List<Rate> __rates = new List<Rate>();
                    Double __feedRateValueNow = __mach.Scripter.GetDRO(21);
                    if (__feedRateValue != __feedRateValueNow)
                    {
                        __feedRateValue = __feedRateValueNow;
                        __rates.Add(new Rate(RateName.Feed, __feedRateValue));
                    }
                    //Double __rapidRateValueNow = __mach.Scripter.GetDRO(21);
                    //if (__rapidRateValue != __rapidRateValueNow)
                    //{
                    //    __rapidRateValue = __rapidRateValueNow;
                    //    __rates.Add(new Rate(RateName.Feed, __rapidRateValue));
                    //}
                    //Double __spindleRateValueNow = __mach.Scripter.GetDRO(21);
                    //if (__spindleRateValue != __spindleRateValueNow)
                    //{
                    //    __spindleRateValue = __spindleRateValueNow;
                    //    __rates.Add(new Rate(RateName.Feed, __spindleRateValue));
                    //}
                    if (__rates.Count != 0)
                    {
                        EventArgs<Rate[]> __e = new EventArgs<Rate[]>(__rates.ToArray());
                        this.RateChanged(__mach, __e);
                    }
                }
                #endregion
                #region SwitchChanged
                if (this.SwitchChanged != null)
                {
                    List<Switch> __switches = new List<Switch>();
                    // map a Scripter method to Up or Down or None
                    SwitchPosition __spindleValueNow = SwitchPosition.Off;
                    if (__mach.Scripter.GetOEMLed(852))
                    {
                        __spindleValueNow = SwitchPosition.One;
                    }
                    if (__mach.Scripter.GetOEMLed(853))
                    {
                        __spindleValueNow = SwitchPosition.Two;
                    }
                    if (__spindleValue != __spindleValueNow)
                    {
                        __spindleValue = __spindleValueNow;
                        __switches.Add(new Switch(SwitchName.Spindle, __spindleValue));
                    }
                    //JogDirection __coolantValueNow = JogDirection.Off;
                    //if (__coolantValue != __coolantValueNow)
                    //{
                    //    __coolantValue = __coolantValueNow;
                    //    __switches.Add(new Switch(SwitchName.Coolant, __coolantValue));
                    //}
                    if (__switches.Count != 0)
                    {
                        EventArgs<Switch[]> __e = new EventArgs<Switch[]>(__switches.ToArray());
                        this.SwitchChanged(__mach, __e);
                    }
                }
                #endregion
            }
			__mach = null;
		}

		internal String GCode
		{
			set
			{
				if (String.IsNullOrEmpty(value))
				{
					this.AutoState = AutoState.Stopped;
				}
				else
				{
					this.AutoState = AutoState.Ready;
				}
			}
		}

		internal AutoState AutoState
		{
			set
			{
				_autoState = value;
			}
		}
	}
}
