using System;
//using System.Windows.Forms;
using System.Collections.Generic;
using System.Text;
using System.IO;

//using System.Diagnostics;
using System.Runtime.InteropServices;

using AKSYSTech.Device.Adapters.Interface;

using ManyLeaves.Infrastructure.Interface;

namespace AKSYSTech.Device.Adapters
{
	public partial class Mach : IDevice
	{
        System.Diagnostics.Process _process = null;
		private Mach4.IMach4 _mach = null;
		private Mach4.IMyScriptObject _scripter = null;
		private Background _background = null;

		#region Events

		public event EventHandler<EventArgs<Position[]>> PositionChanged;
		public event EventHandler<EventArgs<DeviceState>> DeviceStateChanged;
		public event EventHandler<EventArgs<AutoState>> AutoStateChanged;
		public event EventHandler<EventArgs<GCodeState>> GCodeStateChanged;
		public event EventHandler<EventArgs<String>> StatusMessageChanged;
		public event EventHandler<EventArgs<Rate[]>> RateChanged;
        public event EventHandler<EventArgs<Switch[]>> SwitchChanged;



		#endregion

		// used by Background
		internal Mach4.IMyScriptObject Scripter { get { return _scripter; } }

		#region Constructor/Dispose

		public Mach()
		{
		}

		public void Load()
		{
			this.DeviceStateChanged(this, new EventArgs<DeviceState>(DeviceState.Loading));

			// start mach3.exe
			StartMach();
			// attach to mach3
			_mach = (Mach4.IMach4)Marshal.GetActiveObject("Mach4.Document");
			_scripter = (Mach4.IMyScriptObject)_mach.GetScriptDispatch();

			// start background
			_background = new Background(this);
			// hook up using anonymous delegates
			_background.PositionChanged += delegate(Object sender, EventArgs<Position[]> e) { if (this.PositionChanged != null) this.PositionChanged(sender, e); };
			_background.AutoStateChanged += delegate(Object sender, EventArgs<AutoState> e) { if (this.AutoStateChanged != null) this.AutoStateChanged(sender, e); };
			_background.GCodeStateChanged += delegate(Object sender, EventArgs<GCodeState> e) { if (this.GCodeStateChanged != null) this.GCodeStateChanged(sender, e); };
			_background.StatusMessageChanged += delegate(Object sender, EventArgs<String> e) { if (this.StatusMessageChanged != null) this.StatusMessageChanged(sender, e); };
			_background.RateChanged += delegate(Object sender, EventArgs<Rate[]> e) { if (this.RateChanged != null) this.RateChanged(sender, e); };
            _background.SwitchChanged += delegate(Object sender, EventArgs<Switch[]> e) { if (this.SwitchChanged != null) this.SwitchChanged(sender, e); };
			// block until loaded
			_scripter.DoOEMButton(204);
			_scripter.GetMessage();
			// clear EStop
			_scripter.DoButton(21);
			this.DeviceStateChanged(this, new EventArgs<DeviceState>(DeviceState.Loaded));
		}

		public void Close()
		{
			this.DeviceStateChanged(this, new EventArgs<DeviceState>(DeviceState.Closing));
			this.DeviceStateChanged(this, new EventArgs<DeviceState>(DeviceState.Closed));
		}

		public void Dispose()
		{
			// stop background
			_background.Dispose();
			_background = null;

			// detach from mach3

			IntPtr __scripterDispatch = Marshal.GetIDispatchForObject(_scripter);
			Marshal.Release(__scripterDispatch);
			Marshal.FinalReleaseComObject(_scripter);
			_scripter = null;

			IntPtr __machDispatch = Marshal.GetIDispatchForObject(_mach);
			Marshal.Release(__machDispatch);
			Marshal.FinalReleaseComObject(_mach);
			_mach = null;

			// stop mach3.exe
			StopMach();
		}

		private void StopMach()
		{
			_process.CloseMainWindow();
			// I do have sample code to properly close the popup dialogs ... later
			_process.Close();
			_process = null;
		}

		private void StartMach()
		{
            System.Diagnostics.Process[] __processes = System.Diagnostics.Process.GetProcessesByName("Mach3");
			if (__processes.Length == 0)
			{
                System.Diagnostics.ProcessStartInfo __startInfo = new System.Diagnostics.ProcessStartInfo();
				__startInfo.WorkingDirectory = "C:\\Mach3";
				__startInfo.FileName = __startInfo.WorkingDirectory + "\\Mach3.exe";
				__startInfo.Arguments = "/p Mach3Mill";
				//__startInfo.WindowStyle = ProcessWindowStyle.Minimized;
                _process = System.Diagnostics.Process.Start(__startInfo);
			}
			else
			{
				_process = __processes[0];
			}
			_process.WaitForInputIdle();
		}

		#endregion

		#region Properties

		public String GCode
		{
			set
			{
				String __gcodeFileName = Path.GetTempPath() + Path.PathSeparator + Path.GetRandomFileName() + ".ngc";
				File.WriteAllText(__gcodeFileName, value);
				_scripter.LoadFile(__gcodeFileName);
				//File.Delete(__gcodeFileName);
				_background.GCode = value;
			}
		}

		public AutoState AutoState
		{
			set
			{
				switch (value)
				{
                    case AutoState.Ready:
                        if (_scripter.IsEstop() == 1)
                        {
                            _scripter.DoButton(21);
                        }
                        break;
                    case AutoState.Running:
                        _scripter.DoButton(0);
                        break;
                    case AutoState.Paused:
						_scripter.DoButton(1);
						break;
					case AutoState.Stopped:
						_scripter.DoButton(21);
						break;
				}
				_background.AutoState = value;
			}
		}

		#endregion

		#region Methods

		public void Rate(RateName name, Double value)
		{
            switch (name)
            {
                case RateName.Feed:
                    _scripter.SetDRO(21, value);
                    break;
                case RateName.Rapid:
                    break;
                case RateName.Spindle:
                    break;
            }
		}

		public void Rate(RateName name, JogDirection direction)
		{
            switch (name)
            {
                case RateName.Feed:
                    switch (direction)
                    {
                        case JogDirection.Forwards:
                            _scripter.DoOEMButton(108);
                            break;
                        case JogDirection.Backwards:
                            _scripter.DoOEMButton(109);
                            break;
                    }
                    break;
                case RateName.Rapid:
                    break;
                case RateName.Spindle:
                    break;
            }
		}

        public Double Min(RateName name) { return 1; }
        public Double Max(RateName name) { return 300; }

        public void Switch(SwitchName name, SwitchPosition position)
        {
            switch (name)
            {
                case SwitchName.Spindle:
                    switch (position)
                    {
                        case SwitchPosition.Off:
                            _scripter.DoSpinStop();
                            break;
                        case SwitchPosition.One:
                            _scripter.DoSpinCW();
                            break;
                        case SwitchPosition.Two:
                            _scripter.DoSpinCCW();
                            break;
                    }
                    break;
                case SwitchName.Coolant:
                    break;
            }
        }

		public void Jog(Axis axis, JogDirection direction)
		{
			Boolean __jogOn;
			Int16 __axis;
			Int16 __dir;
			switch (direction)
			{
				case JogDirection.Forwards:
					__dir = 0;
					__jogOn = true;
					break;
				case JogDirection.Backwards:
					__dir = 1;
					__jogOn = true;
					break;
				default:
					__dir = -1; // not used
					__jogOn = false;
					break;
			}
			switch (axis)
			{
				case Axis.X:
					__axis = 0;
					break;
				case Axis.Y:
					__axis = 1;
					break;
				case Axis.Z:
					__axis = 2;
					break;
                case Axis.A:
                    __axis = 3;
                    break;
				default:
					__axis = 0;
					break;
			}
			if (__jogOn)
			{
                this.AutoState = AutoState.Manual;
                _scripter.JogOn(__axis, __dir);
            }
			else
			{
                _scripter.JogOff(__axis);
                this.AutoState = AutoState.Ready;
            }
		}

		#endregion
	}
}
