using System;
using System.Collections.Generic;
using System.Text;

using System.Threading;
using System.Runtime.InteropServices;

using AKSYSTech.Device.Adapters.Interface;

using ManyLeaves.Infrastructure.Interface;

namespace AKSYSTech.Device.Adapters
{
	internal class Background : IDisposable
	{
		#region Events

		public event EventHandler<EventArgs<Position[]>> PositionChanged;
		public event EventHandler<EventArgs<AutoState>> AutoStateChanged;
		public event EventHandler<EventArgs<GCodeState>> GCodeStateChanged;
		public event EventHandler<EventArgs<String>> StatusMessageChanged;
        public event EventHandler<EventArgs<Rate[]>> RateChanged;
        public event EventHandler<EventArgs<Switch[]>> SwitchChanged;

		#endregion

		private Thread _thread = null;
		private ManualResetEvent _done = null;

		// we'll need a variable to hold the current value(s) for each xxxChanged event we fire

		Axis _jogAxis;
		JogDirection _direction = JogDirection.Off;
		const Double _jogStep = 0.01;

		Double _maxRate = 1000;
		Double _minRate = 10;
		Double _rateValue = 100;

        SwitchPosition _switchValue = SwitchPosition.Off;

		Double _xValue = 0;
		Double _yValue = 0;
		Double _zValue = 0;

		AutoState _autoState = AutoState.Stopped;
		GCodeState _gcodeState = GCodeState.Completed;

		internal Background(Virtual virtual_)
		{
			_done = new ManualResetEvent(false);
			_thread = new Thread(new System.Threading.ParameterizedThreadStart(DoWork));
			_thread.IsBackground = true;
			_thread.Start(virtual_);
		}

		public void Dispose()
		{
			_done.Set();
			_thread.Join();
		}

		private void DoWork(Object o)
		{
			const Int32 __delay = 50;

			// emulate GCode running by following a circle
			const Double __radius = 10;
			Double __theta = 0;

			Double __xValue = _xValue;
			Double __yValue = _yValue;
			Double __zValue = _zValue;

			AutoState __autoState = _autoState;
			GCodeState __gcodeState = _gcodeState;

			String __statusMessage = String.Empty;

			String __gcode = String.Empty;

			Double __rateValue = 100;
            SwitchPosition __switchValue = SwitchPosition.Off;

			Virtual __virtual_ = (Virtual)o;
			
			while (!_done.WaitOne(__delay, false))
			{
				Double _thetaIncrement = (((Double)_rateValue) * Math.PI) / 6000; // 100% = 3 degrees
				switch (_autoState)
				{
					case AutoState.Ready:
						__theta = 0;
						// simulate jogging
						if (_direction == JogDirection.Off)
						{
						}
						else
						{
							Double __jogStep = _jogStep;
							if (_direction == JogDirection.Backwards)
							{
								__jogStep = -_jogStep;
							}
							switch (_jogAxis)
							{
								case Axis.X:
									__xValue = _xValue + __jogStep;
									break;
								case Axis.Y:
									__yValue = _yValue + __jogStep;
									break;
								case Axis.Z:
									__zValue = _zValue + __jogStep;
									break;
							}
						}
						break;
					case AutoState.Running:
						__theta += _thetaIncrement;
						if (__theta > 2 * Math.PI)
						{
							_gcodeState = GCodeState.Completed;
							_autoState = AutoState.Ready;
						}
						else
						{
							// centre at (-radius, 0)
							_gcodeState = GCodeState.Running;
							__xValue = __radius * (Math.Cos(__theta) - 1);
							__yValue = __radius * Math.Sin(__theta);
						}
						break;
				}
				// position changed?
				if (this.PositionChanged != null)
				{
					List<Position> __positions = new List<Position>();
					if (_xValue != __xValue)
					{
						_xValue = __xValue;
						__positions.Add(new Position(Axis.X, _xValue));
					}
					if (_yValue != __yValue)
					{
						_yValue = __yValue;
						__positions.Add(new Position(Axis.Y, _yValue));
					}
					if (_zValue != __zValue)
					{
						_zValue = __zValue;
						__positions.Add(new Position(Axis.Z, _zValue));
					}
					if (__positions.Count > 0)
					{
						EventArgs<Position[]> __e = new EventArgs<Position[]>(__positions.ToArray());
						this.PositionChanged(__virtual_, __e);
					}
				}
				// auto state changed?
				if (this.AutoStateChanged != null)
				{
					if (__autoState != _autoState)
					{
						if (_autoState == AutoState.Ready)
						{
							__xValue = 0;
							__yValue = 0;
						}
						if (_autoState == AutoState.Stopped)
						{
							if (__autoState == AutoState.Running)
							{
								_gcodeState = GCodeState.Aborted;
							}
						}
						__autoState = _autoState;
						EventArgs<AutoState> __ea = new EventArgs<AutoState>(_autoState);
						this.AutoStateChanged(__virtual_, __ea);
					}
				}
				// gcode state changed?
				if (this.GCodeStateChanged != null)
				{
					if (__gcodeState != _gcodeState)
					{
						__gcodeState = _gcodeState;
						EventArgs<GCodeState> __e = new EventArgs<GCodeState>(_gcodeState);
						this.GCodeStateChanged(__virtual_, __e);
					}
				}
				// status message changed?
				if (this.StatusMessageChanged != null)
				{
					if (__statusMessage != _autoState.ToString())
					{
						__statusMessage = _autoState.ToString();
						EventArgs<String> __e = new EventArgs<String>(__statusMessage);
						this.StatusMessageChanged(__virtual_, __e);
					}
				}
                // feed rate changed
                if (this.RateChanged != null)
                {
                    List<Rate> __rates = new List<Rate>();
                    if (_rateValue != __rateValue)
                    {
                        __rateValue = __rateValue;
                        __rates.Add(new Rate(RateName.Feed, __rateValue));
                    }
                    if (__rates.Count != 0)
                    {
                        EventArgs<Rate[]> __e = new EventArgs<Rate[]>(__rates.ToArray());
                        this.RateChanged(__virtual_, __e);
                    }
                }
                // feed switch changed
                if (this.SwitchChanged != null)
                {
                    List<Switch> __switches = new List<Switch>();
                    if (_switchValue != __switchValue)
                    {
                        _switchValue = __switchValue;
                        __switches.Add(new Switch(SwitchName.Spindle, __switchValue));
                    }
                    if (__switches.Count != 0)
                    {
                        EventArgs<Switch[]> __e = new EventArgs<Switch[]>(__switches.ToArray());
                        this.SwitchChanged(__virtual_, __e);
                    }
                }
            }
			__virtual_ = null;
		}

		internal String GCode
		{
			set
			{
				if (String.IsNullOrEmpty(value))
				{
					this.AutoState = AutoState.Stopped;
				}
				else
				{
					this.AutoState = AutoState.Ready;
					_gcodeState = GCodeState.Loaded;
					_xValue = 0;
					_yValue = 0;
				}
			}
		}

		internal AutoState AutoState
		{
			set
			{
				_autoState = value;
			}
		}

        internal void Rate(RateName name, Double value)
		{
            _rateValue = value;
        }

        internal void Rate(RateName name, JogDirection direction)
        {
            // currently only have a single rate for all Overrides
			const Double _stepRate = 10;
			Double __rateValue = _rateValue;
			if (direction == JogDirection.Forwards)
			{
				if (__rateValue + _stepRate <= _maxRate)
				{
					__rateValue += _stepRate;
				}
				else
				{
					__rateValue = _maxRate;
				}
			}
			if (direction == JogDirection.Backwards)
			{
				if (__rateValue - _stepRate >= _minRate)
				{
					__rateValue -= _stepRate;
				}
				else
				{
					__rateValue = _minRate;
				}
			}
            this.Rate(name, __rateValue);
		}

        internal Double MinRate(RateName name) { return _minRate; }
        internal Double MaxRate(RateName name) { return _maxRate; }

        internal void Switch(SwitchName name, SwitchPosition position)
        {
            _switchValue = position;
        }

		internal void Jog(Axis axis, JogDirection direction)
		{
			_jogAxis = axis;
			_direction = direction;
		}
	}
}
