using System;
using System.Collections.Generic;
using System.Text;

//using System.Diagnostics;
using System.Runtime.InteropServices;

using AKSYSTech.Device.Adapters.Interface;

using ManyLeaves.Infrastructure.Interface;

namespace AKSYSTech.Device.Adapters
{
	public partial class Virtual : IDevice
	{
		private Background _background = null;

		#region Events

		public event EventHandler<EventArgs<Position[]>> PositionChanged;
		public event EventHandler<EventArgs<DeviceState>> DeviceStateChanged;
		public event EventHandler<EventArgs<AutoState>> AutoStateChanged;
		public event EventHandler<EventArgs<GCodeState>> GCodeStateChanged;
		public event EventHandler<EventArgs<String>> StatusMessageChanged;
		public event EventHandler<EventArgs<Rate[]>> RateChanged;
        public event EventHandler<EventArgs<Switch[]>> SwitchChanged;

		#endregion

		#region Constructor/Dispose

		public Virtual()
		{
		}

		public void Load()
		{
			this.DeviceStateChanged(this, new EventArgs<DeviceState>(DeviceState.Loading));
			
			// start background
			_background = new Background(this);
			// hook up using anonymous delegates
			_background.PositionChanged += delegate(Object sender, EventArgs<Position[]> e) { if (this.PositionChanged != null) this.PositionChanged(sender, e); };
			_background.AutoStateChanged += delegate(Object sender, EventArgs<AutoState> e) { if (this.AutoStateChanged != null) this.AutoStateChanged(sender, e); };
			_background.GCodeStateChanged += delegate(Object sender, EventArgs<GCodeState> e) { if (this.GCodeStateChanged != null) this.GCodeStateChanged(sender, e); };
			_background.StatusMessageChanged += delegate(Object sender, EventArgs<String> e) { if (this.StatusMessageChanged != null) this.StatusMessageChanged(sender, e); };
			_background.RateChanged += delegate(Object sender, EventArgs<Rate[]> e) { if (this.RateChanged != null) this.RateChanged(sender, e); };
            _background.SwitchChanged += delegate(Object sender, EventArgs<Switch[]> e) { if (this.SwitchChanged != null) this.SwitchChanged(sender, e); };

			this.DeviceStateChanged(this, new EventArgs<DeviceState>(DeviceState.Loaded));
		}

		public void Close()
		{
			this.DeviceStateChanged(this, new EventArgs<DeviceState>(DeviceState.Closing));
			this.DeviceStateChanged(this, new EventArgs<DeviceState>(DeviceState.Closed));
		}

		public void Dispose()
		{
			// stop background
			_background.Dispose();
			_background = null;
		}

		#endregion

		#region Properties

		public String GCode
		{
			set
			{
				_background.GCode = value;
			}
		}

		public AutoState AutoState
		{
			set
			{
				_background.AutoState = value;
			}
		}

		#endregion

		#region Methods

        public void Rate(RateName name, Double value)
        {
            _background.Rate(name, value);
        }

        public void Rate(RateName name, JogDirection direction)
        {
            _background.Rate(name, direction);
        }

        public Double Min(RateName name) { return _background.MinRate(name); }
        public Double Max(RateName name) { return _background.MaxRate(name); }

        public void Switch(SwitchName name, SwitchPosition position)
        {
            _background.Switch(name, position);
        }

		public void Jog(Axis axis, JogDirection direction)
		{
			_background.Jog(axis, direction);
		}

		#endregion
	}
}
