using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;

using AKSYSTech.Device.Adapters.Interface;
using AKSYSTech.Device.Adapters;

using ManyLeaves.Infrastructure.Interface;

namespace Test
{
	public partial class Form1 : Form
	{
		IDevice _device = null;

		public Form1()
		{
			InitializeComponent();
		}

		private void _loadButton_Click(object sender, EventArgs e)
		{
			_loadButton.Enabled = false;

			IDevice __device = new Virtual();
			if (_machRadioButton.Checked)
			{
				__device = new Mach();
			}
			_device = new Invoke(__device, InvokeDirection.Out, this);

			_device.PositionChanged += new EventHandler<EventArgs<Position[]>>(_device_PositionChanged);
			_device.DeviceStateChanged += new EventHandler<EventArgs<DeviceState>>(_device_DeviceStateChanged);
			_device.AutoStateChanged += new EventHandler<EventArgs<AutoState>>(_device_AutoStateChanged);
			_device.GCodeStateChanged += new EventHandler<EventArgs<GCodeState>>(_device_GCodeStateChanged);
			_device.StatusMessageChanged += new EventHandler<EventArgs<String>>(_device_StatusMessageChanged);
            _device.RateChanged += new EventHandler<EventArgs<Rate[]>>(_device_RateChanged);
            _device.SwitchChanged += new EventHandler<EventArgs<Switch[]>>(_device_SwitchChanged);

			_device.Load();

			_feedRateTrackBar.Minimum = (Int32)_device.Min(RateName.Feed);
            _feedRateTrackBar.Maximum = (Int32)_device.Max(RateName.Feed);
            _device.Rate(RateName.Feed, _feedRateTrackBar.Minimum);
            _device.Rate(RateName.Feed, 100);

            // initialise to aborted
            _runButton.Tag = GCodeState.Aborted;
		}

		private void _loadButton_EnabledChanged(object sender, EventArgs e)
		{
			_unloadButton.Enabled = !_loadButton.Enabled;
			_manualGroupBox.Enabled = _unloadButton.Enabled;
			_autoGroupBox.Enabled = _unloadButton.Enabled;
			_gcodeGroupBox.Enabled = _unloadButton.Enabled;
			_coordinateGroupBox.Enabled = _unloadButton.Enabled;
		}

		void _device_PositionChanged(object sender, EventArgs<Position[]> e)
		{
			foreach (Position __position in e.Data)
			{
				switch (__position.Axis)
				{
					case Axis.X:
						_xLabel.Text = "X: " + __position.Value.ToString("0.000");
						break;
					case Axis.Y:
						_yLabel.Text = "Y: " + __position.Value.ToString("0.000");
						break;
                    case Axis.Z:
                        _zLabel.Text = "Z: " + __position.Value.ToString("0.000");
                        break;
                    case Axis.A:
                        _aLabel.Text = "A: " + __position.Value.ToString("0.000");
                        break;


				}
			}
		}

		void _device_DeviceStateChanged(object sender, EventArgs<DeviceState> e)
		{
			switch (e.Data)
			{
				case DeviceState.Loading:
					_manualGroupBox.Visible = false;
					_autoGroupBox.Visible = false;
					_gcodeGroupBox.Visible = false;
					_coordinateGroupBox.Visible = false;
					break;
				case DeviceState.Loaded:
					_manualGroupBox.Visible = true;
					_autoGroupBox.Visible = true;
					_gcodeGroupBox.Visible = true;
					_coordinateGroupBox.Visible = true;
					break;
				case DeviceState.Closing:
					break;
				case DeviceState.Closed:
					break;
			}
			_deviceStateToolStripStatusLabel.Text = e.Data.ToString();
		}

		void _device_AutoStateChanged(object sender, EventArgs<AutoState> e)
		{
			switch (e.Data)
			{
				case AutoState.Running:
					_manualGroupBox.Enabled = false;
					_runButton.Enabled = false;
					_pauseButton.Enabled = true;
					_continueButton.Enabled = false;
					_stopButton.Enabled = true;
					_resetButton.Enabled = false;
					break;
				case AutoState.Paused:
					_pauseButton.Enabled = false;
					_continueButton.Enabled = true;
					break;
				case AutoState.Stopped:
					_manualGroupBox.Enabled = true;
					_runButton.Enabled = false;
					_pauseButton.Enabled = false;
					_continueButton.Enabled = false;
					_stopButton.Enabled = false;
					_resetButton.Enabled = true;
                    break;
                case AutoState.Ready:
                    _manualGroupBox.Enabled = true;
                    _runButton.Enabled = (((GCodeState)_runButton.Tag) == GCodeState.Loaded);
					_pauseButton.Enabled = false;
					_continueButton.Enabled = false;
					_stopButton.Enabled = true;
					_resetButton.Enabled = false;
					break;
                case AutoState.Manual:
                    _manualGroupBox.Enabled = true;
                    _runButton.Enabled = false;
                    _pauseButton.Enabled = false;
                    _continueButton.Enabled = false;
                    _stopButton.Enabled = true;
                    _resetButton.Enabled = false;
                    break;
            }
			_autoStateToolStripStatusLabel.Text = e.Data.ToString();
		}

		void _device_GCodeStateChanged(object sender, EventArgs<GCodeState> e)
		{
            _runButton.Tag = e.Data;
            switch (e.Data)
			{
				case GCodeState.Loaded:
                    _runButton.Enabled = true;
					break;
				case GCodeState.Aborted:
					break;
				case GCodeState.Completed:
					break;
			}
			_gcodeStateToolStripStatusLabel.Text = e.Data.ToString();
		}

		void _device_StatusMessageChanged(object sender, EventArgs<String> e)
		{
			_statusMessageToolStripStatusLabel.Text = e.Data;
		}

		void _device_RateChanged(object sender, EventArgs<Rate[]> e)
		{
            foreach (Rate __rate in e.Data)
            {
                switch (__rate.Name)
                {
                    case RateName.Feed:
                        if (_feedRateTrackBar.Value != __rate.Value)
                        {
                            _feedRateTrackBar.Value = (Int32)__rate.Value;
                        }
                        _feedRateGroupBox.Text = String.Format("Feed Rate: {0}%", __rate.Value.ToString());
                        break;
                }
            }
		}

        void _device_SwitchChanged(object sender, EventArgs<Switch[]> e)
        {
            foreach (Switch __switch in e.Data)
            {
                switch (__switch.Name)
                {
                    case SwitchName.Spindle:
                        //switch (__switch.Position)
                        //{
                        //    case SwitchPosition.Off:
                        //        _spindleOffRadioButton.Checked = true;
                        //        break;
                        //    case SwitchPosition.One:
                        //        _spindleForwardsRadioButton.Checked = true;
                        //        break;
                        //    case SwitchPosition.Two:
                        //        _spindleBackwardRadioButton.Checked = true;
                        //        break;
                        //}
                        _spindleToolStripStatusLabel.Text = __switch.Position.ToString();
                        break;
                }
            }
        }

		private void _unloadButton_Click(object sender, EventArgs e)
		{
			_loadButton.Enabled = true;

			_device.Close();

			_device.PositionChanged -= new EventHandler<EventArgs<Position[]>>(_device_PositionChanged);
			_device.DeviceStateChanged -= new EventHandler<EventArgs<DeviceState>>(_device_DeviceStateChanged);
			_device.AutoStateChanged -= new EventHandler<EventArgs<AutoState>>(_device_AutoStateChanged);
			_device.StatusMessageChanged -= new EventHandler<EventArgs<String>>(_device_StatusMessageChanged);
			_device.RateChanged -= new EventHandler<EventArgs<Rate[]>>(_device_RateChanged);

			_device.Dispose();
			_device = null;
		}

		private void Form1_FormClosing(object sender, FormClosingEventArgs e)
		{
			if (e.CloseReason != CloseReason.UserClosing) return;
			if (_device == null) return;
			_unloadButton_Click(this, System.EventArgs.Empty);
		}

		private void _jogXNegButton_MouseDown(object sender, MouseEventArgs e)
		{
			_device.Jog(Axis.X, JogDirection.Backwards);
		}

		private void _jogXNegButton_MouseUp(object sender, MouseEventArgs e)
		{
			_device.Jog(Axis.X, JogDirection.Off);
		}

		private void _jogXPosButton_MouseDown(object sender, MouseEventArgs e)
		{
			_device.Jog(Axis.X, JogDirection.Forwards);
		}

		private void _jogXPosButton_MouseUp(object sender, MouseEventArgs e)
		{
			_device.Jog(Axis.X, JogDirection.Off);
        }
        private void _jogYPosButton_MouseDown(object sender, MouseEventArgs e)
        {
            _device.Jog(Axis.Y, JogDirection.Forwards);
        }

        private void _jogYNegButton_MouseDown(object sender, MouseEventArgs e)
        {
            _device.Jog(Axis.Y, JogDirection.Backwards);
        }

        private void _jogYPosButton_MouseUp(object sender, MouseEventArgs e)
        {
            _device.Jog(Axis.Y, JogDirection.Off);
        }

        private void _jogYNegButton_MouseUp(object sender, MouseEventArgs e)
        {
            _device.Jog(Axis.Y, JogDirection.Off);
        }

        private void _jogZPosButton_MouseDown(object sender, MouseEventArgs e)
        {
            _device.Jog(Axis.Z, JogDirection.Forwards);
        }

        private void _jogZPosButton_MouseUp(object sender, MouseEventArgs e)
        {
            _device.Jog(Axis.Z, JogDirection.Off);
        }

        private void _jogZNegButton_MouseDown(object sender, MouseEventArgs e)
        {
            _device.Jog(Axis.Z, JogDirection.Backwards);
        }

        private void _jogZNegButton_MouseUp(object sender, MouseEventArgs e)
        {
            _device.Jog(Axis.Z, JogDirection.Off);
        }

        private void _jogAPosButton_MouseDown(object sender, MouseEventArgs e)
        {
            _device.Jog(Axis.A, JogDirection.Forwards);
        }

        private void _jogAPosButton_MouseUp(object sender, MouseEventArgs e)
        {
            _device.Jog(Axis.A, JogDirection.Off);
        }

        private void _jogANegButton_MouseDown(object sender, MouseEventArgs e)
        {
            _device.Jog(Axis.A, JogDirection.Backwards);
        }

        private void _jogANegButton_MouseUp(object sender, MouseEventArgs e)
        {
            _device.Jog(Axis.A, JogDirection.Off);
        }

		private void _runButton_Click(object sender, EventArgs e)
		{
			_device.AutoState = AutoState.Running;
		}

		private void _pauseButton_Click(object sender, EventArgs e)
		{
			_device.AutoState = AutoState.Paused;
		}

		private void _continueButton_Click(object sender, EventArgs e)
		{
			_device.AutoState = AutoState.Running;
		}

		private void _stopButton_Click(object sender, EventArgs e)
		{
			_device.AutoState = AutoState.Stopped;
		}

		private void _resetButton_Click(object sender, EventArgs e)
		{
			_device.AutoState = AutoState.Ready;
		}

		private void _loadGCodeButton_Click(object sender, EventArgs e)
		{
			_openFileDialog.ShowDialog();
			String __gcodeFileName = _openFileDialog.FileName;
			StreamReader __sr = new StreamReader(__gcodeFileName);
			_gcodeRichTextBox.Text = __sr.ReadToEnd();
			__sr.Close();
		}

		private void _compileGCodeButton_Click(object sender, EventArgs e)
		{
			_device.GCode = _gcodeRichTextBox.Text;
		}

		private void _RateUpButton_Click(object sender, EventArgs e)
		{
            if (sender == _feedRateUpButton)
            {
                _device.Rate(RateName.Feed, JogDirection.Forwards);
            }
		}

		private void _RateDownButton_Click(object sender, EventArgs e)
		{
            if (sender == _feedRateDownButton)
            {
                _device.Rate(RateName.Feed, JogDirection.Backwards);
            }
        }

		private void _RateTrackBar_ValueChanged(object sender, EventArgs e)
		{
            if (sender == _feedRateTrackBar)
            {
                _device.Rate(RateName.Feed, _feedRateTrackBar.Value);
            }
		}

		private void _statusStrip_ItemClicked(object sender, ToolStripItemClickedEventArgs e)
		{

		}

        private void Form1_Load(object sender, EventArgs e)
        {

        }

        private void _spindleRadioButton_CheckedChanged(object sender, EventArgs e)
        {
            if (sender == _spindleOffRadioButton)
            {
                _device.Switch(SwitchName.Spindle, SwitchPosition.Off);
            }
            if (sender == _spindleForwardsRadioButton)
            {
                _device.Switch(SwitchName.Spindle, SwitchPosition.One);
            }
            if (sender == _spindleBackwardsRadioButton)
            {
                _device.Switch(SwitchName.Spindle, SwitchPosition.Two);
            }
        }
        
	}
}